!     $Id: libmtxmpi.F90,v 1.1 2009/10/24 01:12:47 fukuyama Exp $

      MODULE libmpi

#include "finclude/petsc.h"

      integer:: rank,size

      CONTAINS

      SUBROUTINE mtx_initialize(rank_,size_)
      IMPLICIT NONE
      INTEGER,INTENT(OUT):: rank_,size_
      INTEGER:: ierr

      call PetscInitialize(PETSC_NULL_CHARACTER,ierr)
      call MPI_Comm_rank(PETSC_COMM_WORLD,rank,ierr)
      call MPI_Comm_size(PETSC_COMM_WORLD,size,ierr)
      rank_=rank
      size_=size
      return

      END SUBROUTINE mtx_initialize

      SUBROUTINE mtx_finalize
      IMPLICIT NONE
      INTEGER:: ierr

      call PetscFinalize(ierr)
      END SUBROUTINE mtx_finalize

      SUBROUTINE mtx_barrier
      IMPLICIT NONE
      INTEGER:: ierr

      call MPI_Barrier(PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_barrier

      SUBROUTINE mtx_broadcast_character(kdata,n)
      IMPLICIT NONE
      CHARACTER(LEN=n),INTENT(INOUT):: kdata
      INTEGER,INTENT(IN):: n
      INTEGER:: ierr
      
      call mpi_bcast(kdata,n,MPI_CHARACTER,0,PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_broadcast_character

      SUBROUTINE mtx_broadcast_integer(idata,n)
      IMPLICIT NONE
      INTEGER,DIMENSION(n),INTENT(INOUT):: idata
      INTEGER,INTENT(IN):: n
      INTEGER:: ierr
      
      call mpi_bcast(idata,n,MPI_INTEGER,0,PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_broadcast_integer

      SUBROUTINE mtx_broadcast_real8(vdata,n)
      IMPLICIT NONE
      REAL(8),DIMENSION(n),INTENT(INOUT):: vdata
      INTEGER,INTENT(IN):: n
      INTEGER:: ierr
      
      call mpi_bcast(vdata,n,MPI_DOUBLE_PRECISION,0,PETSC_COMM_WORLD,ierr)

      RETURN
      END SUBROUTINE mtx_broadcast_real8

      SUBROUTINE mtx_broadcast_complex8(vdata,n)
      IMPLICIT NONE
      COMPLEX(8),DIMENSION(n),INTENT(INOUT):: vdata
      INTEGER,INTENT(IN):: n
      INTEGER:: ierr
      
      call mpi_bcast(vdata,n,MPI_DOUBLE_COMPLEX,0,PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_broadcast_complex8

      SUBROUTINE mtx_gather_integer(idata,itot)
      IMPLICIT NONE
      INTEGER,INTENT(IN):: idata
      INTEGER,DIMENSION(size),INTENT(OUT):: itot
      INTEGER:: ierr

      call MPI_GATHER(idata,1,MPI_INTEGER,itot,1,MPI_INTEGER,0, &
                      PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_gather_integer

      SUBROUTINE mtx_gather_real8(ddata,dtot)
      IMPLICIT NONE
      REAL(8),INTENT(IN):: ddata
      REAL(8),DIMENSION(size),INTENT(OUT):: dtot
      INTEGER:: ierr

      call MPI_GATHER(ddata,1,MPI_DOUBLE_PRECISION,dtot,1, &
                      MPI_DOUBLE_PRECISION,0,PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_gather_real8

      SUBROUTINE mtx_allgather_integer(idata,itot)
      IMPLICIT NONE
      INTEGER,INTENT(IN):: idata
      INTEGER,DIMENSION(size),INTENT(OUT):: itot
      INTEGER:: ierr

      call MPI_ALLGATHER(idata,1,MPI_INTEGER,itot,1,MPI_INTEGER, &
                         PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_allgather_integer

      SUBROUTINE mtx_gatherv_real8(vdata,ndata,vtot,ntot,ilena,iposa)
      IMPLICIT NONE
      INTEGER,INTENT(IN):: ndata
      INTEGER,INTENT(INOUT):: ntot
      REAL(8),DIMENSION(ndata),INTENT(IN):: vdata
      REAL(8),DIMENSION(ntot),INTENT(OUT):: vtot
      INTEGER,DIMENSION(size):: ilena,iposa
      INTEGER:: ierr

      call MPI_GATHERV(vdata,ndata,MPI_DOUBLE_PRECISION,vtot,ilena,iposa, &
                       MPI_DOUBLE_PRECISION,0,PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_gatherv_real8

      SUBROUTINE mtx_allgatherv_real8(vdata,ndata,vtot,ntot,ilena,iposa)
      IMPLICIT NONE
      INTEGER,INTENT(IN):: ndata
      INTEGER,INTENT(INOUT):: ntot
      REAL(8),DIMENSION(ndata),INTENT(IN):: vdata
      REAL(8),DIMENSION(ntot),INTENT(OUT):: vtot
      INTEGER,DIMENSION(size):: ilena,iposa
      INTEGER:: ierr

      call MPI_ALLGATHERV(vdata,ndata,MPI_DOUBLE_PRECISION, &
                          vtot,ilena,iposa,MPI_DOUBLE_PRECISION, &
                          PETSC_COMM_WORLD,ierr)
      RETURN
      END SUBROUTINE mtx_allgatherv_real8

      END MODULE libmpi
